package net.lecnam.info;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.function.Supplier;

class Arbitrary {

	static Random random = new Random();
	
    // Returns an arbitrary integer
    static Integer arbitraryInteger() {
    	return random.nextInt();
    }

    // Returns an arbitrary integer between 0 (inclusive) and 'bound' (exclusive)
    // throws IllegalArgumentException if bound is less than or equal to 0
    static Integer arbitraryInteger(int bound) { 
    	return random.nextInt(bound);
    }

    // Returns an arbitrary integer between 'start' and 'end' (both inclusive)
    // throws IllegalArgumentException if 'start' is greater than or equal to 'end'
    static Integer arbitraryInteger(int start, int end) {
    	if (start >= end)
    		throw new IllegalArgumentException();
    	int n = end - start;
    	return start + random.nextInt(n + 1);
    }

    // Returns either 'true' or 'false'
    static Boolean arbitraryBoolean() { 
    	return random.nextBoolean();
    }

    // Returns an arbitrary character between 'start' and 'end' (both inclusive)
    // throws IllegalArgumentException if 'start' is greater than or equal to 'end'
    static Character arbitraryCharacter(char start, char end) { 
    	if (start >= end)
    		throw new IllegalArgumentException();
    	int n = end - start;
    	return (char) (start + random.nextInt(n + 1));
    }

    // Returns an arbitrary character that occur in the given string
    // throws IllegalArgumentException if the given string is empty
    static Character arbitraryCharacterFrom(String possibleChars) {
    	int size = possibleChars.length();
    	if (size == 0)
    		throw new IllegalArgumentException();
    	int index = arbitraryInteger(size);
    	return possibleChars.charAt(index);
    }

    // Returns an arbitrary upper case ASCII letter ('A'-'Z')
    static Character arbitraryUppercaseLetter() {
    	return arbitraryCharacter('A', 'Z');
    }

    // Returns an arbitrary lower case ASCII letter ('a'-'z')
    static Character arbitraryLowercaseLetter() {
    	return arbitraryCharacter('a', 'z');    	
    }

    // Returns an arbitrary decimal digit ('0'-'9')
    static Character arbitraryDigit() {
    	return arbitraryCharacter('0', '9');    	    	
    }

    // Returns a string of arbitrary size containing arbitrary chars
    static String arbitraryString(Supplier<Integer> arbitrarySize,
            Supplier<Character> arbitraryCharacter) {
    	Integer size = arbitrarySize.get();
    	String string = "";
    	while (string.length() < size) {
    		Character character = arbitraryCharacter.get();
    		string = string + character;
    	}
    	return string;
    }

    // Returns an arbitrary value taken from the given list
    // throws IllegalArgumentException if the given list is empty
    static <T> T arbitraryFrom(List<T> values) {
    	int size = values.size();
    	if (size == 0)
    		throw new IllegalArgumentException();
    	int index = arbitraryInteger(size);
    	return values.get(index);
    }

    // Returns a list of arbitrary size containing arbitrary items
    static <T> List<T> arbitraryList(Supplier<Integer> arbitrarySize,
            Supplier<T> arbitraryItem) {
    	Integer size = arbitrarySize.get();
    	ArrayList<T> list = new ArrayList<T>();
    	while (list.size() < size) {
    		T item = arbitraryItem.get();
    		list.add(item);
    	}
    	return list;
    }

    // Returns a map of arbitrary size mapping arbitrary keys to arbitrary values
    static <K, V> Map<K, V> arbitraryMap(Supplier<Integer> arbitrarySize,
            Supplier<K> arbitraryKey, Supplier<V> arbitraryValue) {
    	Integer size = arbitrarySize.get();
    	HashMap<K,V> map = new HashMap<K,V>();
    	while (map.size() < size) {
    		K key = arbitraryKey.get();
    		if (!map.containsKey(key)) {
    			V value = arbitraryValue.get();
    			map.put(key, value);
    		}
    	}
    	return map;
    }

    // Returns an array of arbitrary size containing arbitrary strings
    static String[] arbitraryArray(Supplier<Integer> arbitrarySize,
            Supplier<String> arbitraryString) {
    	List<String> list = arbitraryList(arbitrarySize, arbitraryString);
    	String[] array = list.toArray(new String[0]);
    	return array;
    }
}
