package net.lecnam.info;

import static net.lecnam.info.Arbitrary.*;
import static org.junit.jupiter.api.Assertions.*;

import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.CsvFileSource;
import org.junit.jupiter.params.provider.CsvSource;
import org.junit.jupiter.params.provider.MethodSource;

import net.lecnam.info.Calculator;

class CalculatorTest {

	Calculator calc = new Calculator();

	@Test
	@Disabled
	void testAdd() {
		int actual = calc.add(3, 4);
		int expected = 7;
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@CsvSource({ "3, 4, 7", "3, 0, 3", "8, 2, 10" })
	void testAdd(int x, int y, int expected) {
		int actual = calc.add(x, y);
		assertEquals(expected, actual);
	}

	@Test
	@Disabled
	void testDiv1() {
		int actual = calc.div(8, 4);
		int expected = 2;
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@CsvSource({ "12, 4, 3", "0, 3, 0", "8, 2, 4" })
	void testDiv(int x, int y, int expected) {
		int actual = calc.div(x, y);
		assertEquals(expected, actual);
	}

	@Test
	@DisplayName("Testing division by zero")
	void testDiv2() {
		assertThrows(ArithmeticException.class, () -> {
			int actual = calc.div(8, 0);
		});
	}

	@Test
	@Disabled
	void testProd1() {
		int actual = calc.prod(3, 4);
		int expected = 12;
		assertEquals(expected, actual);
	}

	@Test
	@Disabled
	void testProd2() {
		int actual = calc.prod(3, 0);
		int expected = 0;
		assertEquals(expected, actual);
	}

	@Test
	@Disabled
	void testProd3() {
		int actual = calc.prod(0, 2);
		int expected = 0;
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@CsvSource({ "3, 4, 12", "3, 0, 0", "0, 2, 0" })
	void testProd(int x, int y, int expected) {
		int actual = calc.prod(x, y);
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@CsvFileSource(resources = "/testProd2.csv", numLinesToSkip = 1)
	void testProd2(int x, int y, int expected) {
		int actual = calc.prod(x, y);
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@Disabled
	@CsvFileSource(resources = "/testShorter.csv", numLinesToSkip = 1)
	void testShorter(String x, String y, String z, String expected) {
		String actual = calc.shorter(x, y, z);
		assertEquals(expected, actual);
	}

	static IntStream range() {
		return IntStream.range(0, 20).skip(10);
	}

	@ParameterizedTest
	@MethodSource("range")
	void testWithRangeMethodSource(int x) {
		int actual = calc.add(x, x);
		int expected = 2 * x;
		assertEquals(expected, actual);
	}

	@ParameterizedTest
	@CsvFileSource(resources = "/testShorter2.csv", numLinesToSkip = 1)
	void testShorter2(String x, String y, String z) {
		String actual = calc.shorter(x, y, z);
		assertTrue(actual.length() <= x.length() && actual.length() <= y.length() && actual.length() <= z.length()
				&& (actual == x || actual == y || actual == z));
	}

	static IntStream random() {
		return new Random().ints(0, 100).limit(20);
	}

	@ParameterizedTest(name = "Trying {0}")
	@MethodSource("random")
	void testadd3(int x) {
		int actual = calc.add(x, x);
		int expected = 2 * x;
		assertEquals(expected, actual);
	}

	static Stream<Integer> testCaseProviderForAdd() {
		return Stream.generate(() -> arbitraryInteger(100)).limit(20);
	}

	@ParameterizedTest(name = "Trying {0}")
	@MethodSource("testCaseProviderForAdd")
	void testadd4(int x) {
		int actual = calc.add(x, x);
		int expected = 2 * x;
		assertEquals(expected, actual);
	}

	static String arbitraryLowercaseString(int bound) {
		return arbitraryString(() -> arbitraryInteger(bound), () -> arbitraryLowercaseLetter());
	}

	static Arguments argumentsForShorter() {
		return Arguments.of(arbitraryLowercaseString(10), arbitraryLowercaseString(10), arbitraryLowercaseString(10));
	}

	static Stream<Arguments> testCaseProviderForShorter() {
		return Stream.generate(() -> argumentsForShorter()).limit(20);
	}

	@ParameterizedTest
	@MethodSource("testCaseProviderForShorter")
	void testShorter3(String x, String y, String z) {
		String actual = calc.shorter(x, y, z);
		assertTrue(actual.length() <= x.length() && actual.length() <= y.length() && actual.length() <= z.length()
				&& (actual == x || actual == y || actual == z));
	}

	static List<String> arbitraryStringList(int bound) {
		return arbitraryList(() -> arbitraryInteger(bound), () -> arbitraryLowercaseString(10));
	}

	static Stream<List<String>> testCaseProviderForSort() {
		return Stream.generate(() -> arbitraryStringList(10)).limit(20);
	}

	@ParameterizedTest
	@MethodSource("testCaseProviderForSort")
	void testSort(List<String> list) {
		String[] array = list.toArray(new String[0]);
		String[] array2 = list.toArray(new String[0]);
		Calculator.sort(array);
		Arrays.sort(array2);
		assertArrayEquals(array2, array);
	}

}
